/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
DSP effects for LiveLight V2 Firmware 						 		 ________   __
Written by Henri Rantanen											|  ____\ \ / /
																	| |__   \ V / 
Date		15.1.2012												|  __|   > <  
Version		2.4														| |     / . \ 
Target		PCB Version 5, Atmel ATMEGA32-16U						|_|    /_/ \_\

************************************************************************************/
#include "settings.h"


void ResetShutdownTimer(void)					// Reset time to Off-line
{
	if (TIMSK & ~0x01) TIMSK |= (1 << TOIE0);	// Re-enable shut-down timer after "Off-line"

	TCNT0 = 0;									// Reset Shut-Down timer
}


ISR(TIMER0_OVF_vect)							// TIMER0 Overflow interrupt
{	
	static uint8_t NoInputCounter = 0;			// Off-line timer calculator
	
	if (NoInputCounter > SLEEP_DELAY)			// If counter has reached max. value
	{
		TIMSK = TIMSK & ~(1 << TOIE0);			// Disable TIMER0 interrupt
		
		NoInputCounter = 0;						// Reset shutdown counter	

		FadeOut();								// Fade lights OFF
	}
	
	else NoInputCounter++;						// Grow Off-line counter
}


void Color(uint8_t Red, uint8_t Green, uint8_t Blue)  // Set RGB color to registers
{
	Compare[1] = RXBuffer[1] = Red; 
	Compare[2] = RXBuffer[2] = Green;
	Compare[3] = RXBuffer[3] = Blue;
	Compare[4] = RXBuffer[4] = Red;
	Compare[5] = RXBuffer[5] = Green;
	Compare[6] = RXBuffer[6] = Blue;
	Compare[7] = RXBuffer[7] = Red;
	Compare[8] = RXBuffer[8] = Green;
	Compare[9] = RXBuffer[9] = Blue;
	Compare[10] = RXBuffer[10] = Red;
	Compare[11] = RXBuffer[11] = Green;
	Compare[12] = RXBuffer[12] = Blue;
	Compare[13] = RXBuffer[13] = Red;
	Compare[14] = RXBuffer[14] = Green;
	Compare[15] = RXBuffer[15] = Blue;
	Compare[16] = RXBuffer[16] = Red;
	Compare[17] = RXBuffer[17] = Green;
	Compare[18] = RXBuffer[18] = Blue;
	Compare[19] = RXBuffer[19] = Red;
	Compare[20] = RXBuffer[20] = Green;
	Compare[21] = RXBuffer[21] = Blue;
	Compare[22] = RXBuffer[22] = Red;
	Compare[23] = RXBuffer[23] = Green;
	Compare[24] = RXBuffer[24] = Blue;
	Compare[25] = RXBuffer[25] = Red;
	Compare[26] = RXBuffer[26] = Green;
	Compare[27] = RXBuffer[27] = Blue;	
}


void FadeOut(void)
{
	volatile uint8_t i;
	volatile uint8_t j;
	volatile uint8_t h;
	
	cli();	// Disable all other functions
	
	for (j = 0; j < 9; j++)
	{		
		// Desaturate the RGB channel
		RXBuffer[j*3+1] = RXBuffer[j*3+2] = RXBuffer[j*3+3] = ((RXBuffer[j*3+1] + RXBuffer[j*3+2] + RXBuffer[j*3+3]) / 3);
		Compare[j*3+1] = Compare[j*3+2] = Compare[j*3+3] = ((Compare[j*3+1] + Compare[j*3+2] + Compare[j*3+3]) / 3);
	}
	
	switch_routine(); //Update values to PWM

	for (i = 0; i < 255; i++)
	{
			// Dim the RGB channel
			for (j = 0; j<=Channels; j++)
			{
				if (RXBuffer[j] > 0)
				{
					RXBuffer[j]--;
				}
			}
		
			// Run full PWM cycle
			for (h = 0; h < 5; h++)
			{
				for (j = 0; j < 255; j++) switch_routine();
			}
/*for (j = 0; j < 255; j++) switch_routine();*/
	}
	
	Color(0,0,0);
	
	sei(); // Enable all interrupts
	
}