/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
LiveLight USB Interface with ATMEGA32
Written by Henri Rantanen 

Date		15.1.2012
Version		2.4
Target		PCB Version 5, Atmel ATMEGA32-16U

************************************************************************************/
#include "settings.h"

int main(void)
{	 
	// Initialize the MCU
	Initialize();
		
	while(1)
    {
		// Main loop is empty
 		// Interrupts do all the work!
    }
}

void Initialize(void)
{
		// Read first byte from EEPROM and check if memory is empty or damaged.
	if (EE_HEADER != 0xDB)
	{
		WriteEE_defaults();
	}
	
	// Initialize TIMER0							// Sleep Timer
	TCCR0 |= (1 << CS02)|(0 << CS01)|(0 << CS00);	// Timer pre-scaler 1/1024
	TIMSK |= (1 << TOIE0);							// Enable Overflow Interrupt
	TCNT0 = 0;										// Initialize Counter
	
	// Initialize TIMER2							// PWM Timer
	TCCR2 |= (0 << CS22)|(1 << CS21)|(0 << CS20);	// Timer pre-scaler 1/8
	TIMSK |= (1 << TOIE2);							// Enable Overflow Interrupt
	TCNT2 = 0;										// Initialize Counter
	
	//Initialize USART
	UBRRH = 0;										// UART Speed HIGH
	UBRRL = ((F_CPU / 16) / (BAUD_RATE * 100)) - 1; // UART Speed LOW
	UCSRB |= (1 << RXEN )|(1 << TXEN )|(1 << RXCIE);// Enable RX, TX and RX Interrupt
	UCSRC |= (1 << URSEL)|(1 << UCSZ1)|(1 << UCSZ0);// Character size 8 bits

	// Initialize output pins
	DDRA = 0xFF;									// PORT A 1111 1111			
	DDRB = 0xFF;									// PORT B 1111 1111
	DDRC = 0xFF;									// PORT C 1111 1111
	//DDRD = 0xFE;									// PORT D 1111 1110 (For PCB V4)
	DDRD = 0xFC;									// PORT D 1111 1100
	
	sei();											// Enable global interrupts
}