/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
PWM Algorithm for LiveLight V2 Firmware					 _______          ____  __ 
Written by Henri Rantanen								|  __ \ \        / /  \/  |
														| |__) \ \  /\  / /| \  / |
DDate		15.1.2012									|  ___/ \ \/  \/ / | |\/| |
Version		2.4											| |      \  /\  /  | |  | |
Target		PCB Version 5, Atmel ATMEGA32-16U			|_|       \/  \/   |_|  |_|

************************************************************************************/
#include "settings.h"

// TIMER2 Overflow interrupt
ISR(TIMER2_OVF_vect)
{
	switch_routine();
	switch_routine();
	
	TCNT2 = 200;	// Set the starting value of the PWM timer	
}

void switch_routine(void)
{
	static uint8_t PWM_Index = 0xFF;	// PWM Index counter
	static uint8_t PinMaskA = 0xFF;		// Port A Output Pattern
	static uint8_t PinMaskB = 0xFF;		// Port B Output Pattern
	static uint8_t PinMaskC = 0xFF;		// Port C Output Pattern
	static uint8_t PinMaskD = 0xFC;		// Port D Output Pattern
	
	// Update outputs
	PORTA = PinMaskA;					// Set A port to calculated pattern
	PORTB = PinMaskB;					// Set B port to calculated pattern
	PORTC = PinMaskC;					// Set C port to calculated pattern
	PORTD = PinMaskD;					// Set D port to calculated pattern
	
	if (++PWM_Index == 0)				// Update colors from RX buffer
	{
		PinMaskA = 0xFF;				// Set Port A ON
		PinMaskB = 0xFF;				// Set Port B ON
		PinMaskC = 0xFF;				// Set Port C ON
		PinMaskD = 0xFC;				// Set Port D ON except TX & RX pins
		
		Compare[1] = RXBuffer[1];		// Update channels from receiver buffer 
		Compare[2] = RXBuffer[2];
		Compare[3] = RXBuffer[3];
		Compare[4] = RXBuffer[4];
		Compare[5] = RXBuffer[5];
		Compare[6] = RXBuffer[6];
		Compare[7] = RXBuffer[7];
		Compare[8] = RXBuffer[8];		// No loop? This runs faster!
		Compare[9] = RXBuffer[9];
		Compare[10] = RXBuffer[10];
		Compare[11] = RXBuffer[11];
		Compare[12] = RXBuffer[12];
		Compare[13] = RXBuffer[13];
		Compare[14] = RXBuffer[14];
		Compare[15] = RXBuffer[15];
		Compare[16] = RXBuffer[16];
		Compare[17] = RXBuffer[17];
		Compare[18] = RXBuffer[18];
		Compare[19] = RXBuffer[19];
		Compare[20] = RXBuffer[20];
		Compare[21] = RXBuffer[21];
		Compare[22] = RXBuffer[22];
		Compare[23] = RXBuffer[23];
		Compare[24] = RXBuffer[24];
		Compare[25] = RXBuffer[25];
		Compare[26] = RXBuffer[26];
		Compare[27] = RXBuffer[27];
	}
	
	// Remove channels from the pattern used in next interrupt
	if(Compare[1] == PWM_Index)		PinMaskA &= ~(1 << PA2); //R
	if(Compare[2] == PWM_Index)		PinMaskA &= ~(1 << PA4); //G
	if(Compare[3] == PWM_Index)		PinMaskA &= ~(1 << PA3); //B
	if(Compare[4] == PWM_Index)		PinMaskB &= ~(1 << PB2); //R
	if(Compare[5] == PWM_Index)		PinMaskA &= ~(1 << PA1); //G
	if(Compare[6] == PWM_Index)		PinMaskB &= ~(1 << PB1); //B
	if(Compare[7] == PWM_Index)		PinMaskA &= ~(1 << PA0); //R
	if(Compare[8] == PWM_Index)		PinMaskB &= ~(1 << PB3); //G
	if(Compare[9] == PWM_Index)		PinMaskB &= ~(1 << PB0); //B
	if(Compare[10] == PWM_Index)	PinMaskD &= ~(1 << PD3); //R
	if(Compare[11] == PWM_Index)	PinMaskB &= ~(1 << PB4); //G
	if(Compare[12] == PWM_Index)	PinMaskD &= ~(1 << PD2); //B
	if(Compare[13] == PWM_Index)	PinMaskD &= ~(1 << PD4); //R
	if(Compare[14] == PWM_Index)	PinMaskD &= ~(1 << PD6); //G
	if(Compare[15] == PWM_Index)	PinMaskD &= ~(1 << PD5); //B
	if(Compare[16] == PWM_Index)	PinMaskC &= ~(1 << PC0); //R
	if(Compare[17] == PWM_Index)	PinMaskD &= ~(1 << PD7); //G
	if(Compare[18] == PWM_Index)	PinMaskC &= ~(1 << PC1); //B
	if(Compare[19] == PWM_Index)	PinMaskC &= ~(1 << PC4); //R
	if(Compare[20] == PWM_Index)	PinMaskC &= ~(1 << PC2); //G
	if(Compare[21] == PWM_Index)	PinMaskC &= ~(1 << PC3); //B
	if(Compare[22] == PWM_Index)	PinMaskC &= ~(1 << PC5); //R
	if(Compare[23] == PWM_Index)	PinMaskC &= ~(1 << PC7); //G
	if(Compare[24] == PWM_Index)	PinMaskC &= ~(1 << PC6); //B
	if(Compare[25] == PWM_Index)	PinMaskA &= ~(1 << PA5); //R
	if(Compare[26] == PWM_Index)	PinMaskA &= ~(1 << PA6); //G
	if(Compare[27] == PWM_Index)	PinMaskA &= ~(1 << PA7); //B
}