/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
USART RX Input code for LiveLight V2 Firmware 						 _____  __   __
Written by Henri Rantanen											|  __ \ \ \ / /
																	| |__) | \ V /
Date		15.1.2012												|  _  /   > <  
Version		2.4														| | \ \  / . \ 
Target		PCB Version 5, Atmel ATMEGA32-16U						|_|  \_\/_/ \_\

************************************************************************************/
#include "settings.h"

/* Data input protocol example
	+--------+-------+---------+--------+-------+---------+--------+-------+----------/
	| Header | Red 1 | Green 1 | Blue 1 | Red 2 | Green 2 | Blue 2 | Red 3 | Green3  /
	+--------+-------+---------+--------+-------+---------+--------+-------+--------/
*/

ISR(USART_RXC_vect)							// UART Receive interrupt
{	
	static volatile uint8_t i = 0;			// Index variable
	const uint8_t HeaderByte = HEADER_BYTE;	// Read header byte from EEPROM
	const uint8_t BlackLevel = BLACK_LEVEL;	// Read black level from EEPROM
	volatile uint8_t buffer = 0;			// Receiver buffer
	
	buffer = UDR;							// Read USART to buffer
	
	ResetShutdownTimer();					// Reset shutdown timer
	
 	if (i > Channels || RXBuffer[0] != HeaderByte) i = 0;	// Reset index						

	if (i == 0)	RXBuffer[0] = buffer;		// Read header byte
	
	else									// Read data to use
	{
		if (buffer < BlackLevel) RXBuffer[i] = BlackLevel; // Apply minimum brightness
		
		else RXBuffer[i] = Average(RXBuffer[i], buffer);	// Calculate moving average of the 
	}

	if (RXBuffer[0] == HeaderByte) i++;		// Grow index if header is OK
}
 
uint8_t Average(uint32_t old, uint32_t new)
{		
		// Agressive attack if change is great enough
		//if (new == 0x00) return new;	// Skip to black
		//if (new == 0xFF) return new;	// Skip to white
		
		if (new < 0xFF)	new = ((old << 7) + (new << 7)) >> 8;
		if (new < 0x80)	new = ((old << 7) + (new << 7)) >> 8;
		if (new < 0x40)	new = ((old << 7) + (new << 7)) >> 8;
		if (new < 0x20)	new = ((old << 7) + (new << 7)) >> 8;
		 				
		return new;
}

// ISR(USART_RXC_vect)							// UART Receive interrupt
// {	
// 	static volatile uint8_t i = 0;			// Index variable
// 	const uint8_t HeaderByte = HEADER_BYTE;
// 	const uint8_t BlackLevel = BLACK_LEVEL;
// 	//volatile uint16_t buff = 0;
// 	
// 	int *buffer = malloc(1);				// Allocate byte of memory
// 	
// 	*buffer = UDR;
// 	
// 	ResetShutdownTimer();					// Reset shutdown timer
// 	
//  	if (i > Channels || RXBuffer[0] != HeaderByte) i = 0;				// Reset index						
// 
// 	if (i == 0)	RXBuffer[0] = *buffer;		// Read header byte
// 	else
// 	{
// 		if (*buffer < BlackLevel) RXBuffer[i] = BlackLevel;
// 		
// 		else RXBuffer[i] = Average(RXBuffer[i], *buffer);
// 	}
// 	
// 	free(buffer);
// 
// 	if (RXBuffer[0] == HeaderByte) i++;		// Grow index if header is OK
// }